//
//  TSUserDefaultData.swift
//  AIRingtone
//
//  Created by 100Years on 2025/3/5.
//
import ObjectMapper

func kHandleTSHistory(){
    for model in TSAIRintoneHistory.shared.listModels {
        if model.modelType != .example {
            switch model.actionStatus {
            case .pending,.running:
                TSGenerateRintoneOperationQueue.shared.creatOperation(uuid: model.uuid).getActionInfo(oldModel: model)
            default:break
            }
        }
    }
    
    for model in TSPosterHistory.shared.listModels {
        if model.modelType != .example {
            switch model.actionStatus {
            case .pending,.running:
                TSGeneratePosterOperationQueue.shared.creatOperation(uuid: model.uuid).getActionInfo(oldModel: model)
            default:break
            }
        }
    }
    
    for model in TSPhotoHistory.shared.listModels {
        if model.modelType != .example {
            switch model.actionStatus {
            case .pending,.running:
                TSGeneratePhotoOperationQueue.shared.creatOperation(uuid: model.uuid).getActionInfo(oldModel: model)
            default:break
            }
        }
    }
}

// MARK: - 基础历史记录类
class TSBaseHistoryManager<ModelType: TSBaseModel> {
    // 子类必须重写的属性
    var historyKey: String { fatalError("必须重写 historyKey") }
    var exampleDataKey: String { fatalError("必须重写 exampleDataKey") }
    var exampleModels: [ModelType] { fatalError("必须重写 exampleModels") }

    func findModelID(modelID: Int)->Int?{
        fatalError("必须重写 findModelID")
    }

    func saveModelAfterProcess(){
        
    }
    
    // 存储属性
    private var _historyString: String {
        get { UserDefaults.standard.string(forKey: historyKey) ?? "" }
        set { UserDefaults.standard.set(newValue, forKey: historyKey) }
    }
    
    private var _listModels: [ModelType]?
    var listModels: [ModelType] {
        get {
            if _listModels == nil { loadModels() }
            return _listModels ?? []
        }
        set {
            _listModels = newValue
        }
    }
    
    // MARK: - 公共方法
    func saveModel(model: ModelType, at index: Int = 0) {
        listModels.insert(model, at: index)
        saveHistory()
        saveModelAfterProcess()
    }
    
    func removeModel(model:ModelType) {
        self.listModels.removeAll { $0 === model }
        saveHistory()
    }
    
    func removeModel(index: Int) {
        guard index >= 0 && index < listModels.count else { return }
        listModels.remove(at: index)
        saveHistory()
    }
    
    func replaceModel(oldID: Int, newModel: ModelType){
        if let index = findModelID(modelID: oldID) {
            listModels[index] = newModel
            dePrint("\(Self.self).listModels Model replaced at index \(index)")
        } else {
            listModels.insert(newModel, at: 0)
            dePrint("\(Self.self).listModels Model not found")
        }
        dePrint("\(Self.self).listModels.count=\(listModels.count)")
        saveHistory()
    }
    
    func replaceAndSaveModel(saveModel:ModelType,compareBlock:(ModelType,ModelType)->Bool){
        if let index = listModels.firstIndex(where: { model in
            compareBlock(saveModel,model)
        }){
            dePrint("\(Self.self).listModels Model replaced at index \(index)")
           listModels[index] = saveModel
            saveHistory()
        }else{
            self.saveModel(model: saveModel)
        }
    }
    
    func dePrintAllModel() {
        dePrint("=======================结果查询开始======================")
        dePrint("\(Self.self).listModels.count=\(listModels.count)")
        for model in listModels {
            dePrint(model.toJSON())
        }
        dePrint("=======================结果查询结束======================")
    }
    
    // MARK: - 私有方法
    private func saveHistory() {
        if let jsonString = listModels.toJSONString() {
            _historyString = jsonString
        }
    }
    
    private func loadModels() {
        if exampleModels.count > 0 {
            // 第一次运行时插入示例数据
            if UserDefaults.standard.string(forKey: exampleDataKey) == nil {
                insertExampleData()
                UserDefaults.standard.set("1", forKey: exampleDataKey)
            }
        }

        // 从历史记录加载模型
        if let models = Mapper<ModelType>().mapArray(JSONString: _historyString) {
            _listModels = models
        } else {
            _listModels = []
        }
    }
    
    private func insertExampleData() {
        if let jsonString = exampleModels.toJSONString() {
            _historyString = jsonString
        }
    }
}

// MARK: - 海报历史记录
final class TSPosterHistory: TSBaseHistoryManager<TSActionInfoModel> {
    static let shared = TSPosterHistory()
    override var historyKey: String { "kPosterTextPicHistoryListString" }
    override var exampleDataKey: String { "insertPosterExampleData" }
    
    override func findModelID(modelID: Int) -> Int? {
        return listModels.firstIndex(where: {$0.id == modelID})
    }
    
    override var exampleModels: [TSActionInfoModel] {
        [
            createExampleModel(imageName: "poster_example_0"),
            createExampleModel(imageName: "poster_example_1"),
            createExampleModel(imageName: "poster_example_2")
        ]
    }
    
    private func createExampleModel(imageName: String) -> TSActionInfoModel {
        let model = TSActionInfoModel()
        model.modelType = .example
        model.request.prompt = "Example"
        model.request.promptSort = "Example"
        model.request.width = kTextPicW
        model.request.height = kTextPicH
        model.response.resultUrl = imageName
        return model
    }
}
// MARK: - 头像历史记录
final class TSPhotoHistory: TSBaseHistoryManager<TSActionInfoModel> {
    static let shared = TSPhotoHistory()
    override var historyKey: String { "kPhotoTextPicHistoryListString" }
    override var exampleDataKey: String { "insertPhotoExampleData" }
    
    override func findModelID(modelID: Int) -> Int? {
        return listModels.firstIndex(where: {$0.id == modelID})
    }
    
    override var exampleModels: [TSActionInfoModel] {
        [
            createExampleModel(imageName: "photo_example_0"),
            createExampleModel(imageName: "photo_example_1"),
            createExampleModel(imageName: "photo_example_2")
        ]
    }
    
    private func createExampleModel(imageName: String) -> TSActionInfoModel {
        let model = TSActionInfoModel()
        model.modelType = .example
        model.request.prompt = "Example"
        model.request.promptSort = "Example"
        model.request.width = kTextPicW
        model.request.height = kTextPicW
        model.response.resultUrl = imageName
        return model
    }
}
// MARK: - AI铃声历史记录
final class TSAIRintoneHistory: TSBaseHistoryManager<TSActionInfoModel> {
    static let shared = TSAIRintoneHistory()
    override var historyKey: String { "kRintoneTextMusicHistoryListString" }
    override var exampleDataKey: String { "insertRintoneExampleData" }
    
    override func findModelID(modelID: Int) -> Int? {
        return listModels.firstIndex(where: {$0.id == modelID})
    }
    
    override var exampleModels: [TSActionInfoModel] {
        [
            createExampleModel()
        ]
    }
    
    private  func createExampleModel()->TSActionInfoModel{
        let model = TSActionInfoModel()
        model.modelType = .example
        model.request.duration = 30
        model.request.prompt = "Example"
        model.request.promptSort = "Example"
        model.response.coverUrl = "http://d3a93z8fj970a4.cloudfront.net/53337c41-9610-4b92-a8fc-0871a0895880"
        model.response.title = "Glass"
        model.response.musicUrl = "http://d3a93z8fj970a4.cloudfront.net/328324a2-2576-4f55-8c77-2fa16b88e043"
        return model
    }
}
// MARK: - 我的下载铃声历史记录
final class TSMineRintoneHistory: TSBaseHistoryManager<TSRingModel> {
    static let shared = TSMineRintoneHistory()
    override var historyKey: String { "kMineRintoneHistoryListString" }
    override var exampleDataKey: String { "insertMineRintoneExampleData" }
    
    var isHaveNew:Bool = false
    
    override func findModelID(modelID: Int) -> Int? {
        return nil
    }
    
    override var exampleModels: [TSRingModel] {
        []
    }
    
    override func saveModelAfterProcess() {
        isHaveNew = true
    }
}

