//
//  TSAIRintoneHistoryCell.swift
//  AIRingtone
//
//  Created by 100Years on 2025/3/6.
//
import Combine
import SwipeCellKit
class TSAIRintoneHistoryCell: SwipeCollectionViewCell  {

    weak var targetVC:UIViewController?
    var indexPath:IndexPath = IndexPath(item: 0, section: 0)
    override init(frame: CGRect) {
        super.init(frame: frame)
        creatUI()
        dealThings()
    }
    
    required public init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    static let cellID = "TSAIRintoneHistoryCell"

    lazy var ringView: TSRingToneCellView = {
        let ringToneView = TSRingToneCellView()
        ringToneView.cornerRadius = 0.0
        ringToneView.setRingBtn.isHidden = false
        ringToneView.editBtn.isHidden = false
        ringToneView.clickPlayHandel = { [weak self] isplay in
            guard let self = self else { return }
            clickPlay()
        }
        
        ringToneView.clickEditHandel = { [weak self] in
            guard let self = self else { return }
            clickEidtBtn()
        }
        
        ringToneView.clickSetUpHandel = { [weak self] in
            guard let self = self else { return }
            clickSetRingBtn()
      
        }
        return ringToneView
    }()
    
    lazy var generateView: TSRingToneGenerateView = {
        let generateView = TSRingToneGenerateView()
        generateView.isHidden = true
        generateView.refreshHandel = { [weak self]  in
            guard let self = self else { return }
            guard let oldModel = model else { return }
            if kPurchaseToolShared.kJudgeVipFreeType(vipFreeNumType: .ringtones){ return }//判断 vip
            TSGenerateRintoneOperationQueue.shared.creatOperation(uuid: oldModel.uuid).creatRintone(oldModel: oldModel, prompt: oldModel.request.prompt, promptSort: oldModel.request.promptSort)
        }
        return generateView
    }()
    
    
    lazy var exampleView: UIView = {
        let exampleView = UIView()
        exampleView.backgroundColor = "#7E57F4".uiColor
        
        let textLabel = UILabel.createLabel(
            text: "Example".localized,
            font: .font(size: 12),
            textColor: .white
        )
        
        exampleView.addSubview(textLabel)
        textLabel.snp.makeConstraints { make in
            make.top.edges.equalTo(UIEdgeInsets(top: 4, left: 6, bottom: 4, right: 6))
        }
        
        kDelayMainShort {
            exampleView.makeCorner([.topLeft,.bottomLeft], radius: 10)
        }
        return exampleView
    }()
    
    
    lazy var vipView: UIImageView = {
        let vipView = UIImageView.createImageView(imageName: "ring_vip")
        vipView.isHidden = true
        return vipView
    }()
    
    var model:TSActionInfoModel?{
        didSet{
            modelDidSet()
        }
    }

    var ringModel:TSRingModel?{
        didSet{
            ringModelDidSet()
        }
    }
     
    func creatUI() {
        cornerRadius = 16.0
        backgroundColor = .cardColor
        
        contentView.addSubview(ringView)
        ringView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
    
        contentView.addSubview(vipView)
        vipView.snp.makeConstraints { make in
            make.top.trailing.equalTo(0)
            make.width.height.equalTo(24)
        }
        
        contentView.addSubview(exampleView)
        exampleView.snp.makeConstraints { make in
            make.trailing.equalToSuperview()
            make.top.equalTo(0)
            make.height.equalTo(20)
        }
  
        contentView.addSubview(generateView)
        generateView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
    }
    
    func setTargetVC(targetVC:UIViewController?,indexPath:IndexPath) {
        self.targetVC = targetVC
        self.indexPath = indexPath
        
        if let vc = targetVC as? any SwipeCollectionViewCellDelegate {
          self.delegate = vc
        }
    }
    
    func dealThings(){
//        NotificationCenter.default.addObserver(forName: .kBusinessAudioStateChange, object: nil, queue: nil) { notification in
//            if let userInfo = notification.userInfo as? [String: TSBusinessAudioPlayer.PlayerState], let state = userInfo["PlayerState"] {
//                kExecuteOnMainThread {
//                    self.changePlayerState(state: state)
//                }
//            }
//        }
    }
    
    override func prepareForReuse() {
        super.prepareForReuse()
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }

}


extension TSAIRintoneHistoryCell {
    
    func modelDidSet(){
        guard let model = model else { return }
        if let rintoneOperation = TSGenerateRintoneOperationQueue.shared.findOperation(uuid: model.uuid) as? TSGenerateRintoneOperation {
            rintoneOperation.currentActionInfoModelChanged = { [weak self] actionInfoModel in
                guard let self = self else { return }
                DispatchQueue.main.async {
                    self.updataActionInfoModelView(model: actionInfoModel)
                }
            }
        }

        updataActionInfoModelView(model: model)
        dePrint("model actionStatus 收到=\(model.actionStatus)")
        
        if playSelf{
            setPlayerStateChangedHandle()
        }else{
            changePlayerState(state: .stop)
        }
    }
    
    func ringModelDidSet(){
        guard let ringModel = ringModel else { return }

        ringView.timeLab.text = Float(ringModel.duration).floatToMinuteSecond()
        ringView.nameLab.text = ringModel.title
        ringView.setCoverImageView(urlString: "")
        exampleView.isHidden = true
        generateView.isHidden = true
        vipView.isHidden = !ringModel.vip
        
        if playSelf{
            setPlayerStateChangedHandle()
        }else{
            changePlayerState(state: .stop)
        }
    }
    
    func setPlayerStateChangedHandle(){
        
        TSBusinessAudioPlayer.shared.stateChangedHandle = { [weak self] playerState in
            guard let self = self else { return }
            self.changePlayerState(state: playerState)
        }
        self.changePlayerState(state: TSBusinessAudioPlayer.shared.currentPlayerState)
    }
    
}


extension TSAIRintoneHistoryCell {
    func clickPlay() {
        if playSelf{
            if TSBusinessAudioPlayer.shared.isLoading{
                return
            }
            if TSBusinessAudioPlayer.shared.isPlaying{
                TSBusinessAudioPlayer.shared.stop()
            }else{
                TSBusinessAudioPlayer.shared.playUrlString(modelUrlString,localURL: modelLocalURL,indexPath: indexPath)
            }
            
        }else{
            TSBusinessAudioPlayer.shared.stop()//让上一个对象停止播放
            setPlayerStateChangedHandle()//更换监听
            TSBusinessAudioPlayer.shared.playUrlString(modelUrlString,localURL: modelLocalURL,indexPath: indexPath)
        }
    }
    

    func changePlayerState(state:TSBusinessAudioPlayer.PlayerState){
//        if playSelf == false {
//            if ringView.isPlay == true ||
//                ringView.isloading == true
//            {
//                ringView.handelAudioPlayerStateChange(state: .stop)
//            }
//            return
//        }
//        dePrint("changePlayerState = \(state)")
        ringView.handelAudioPlayerStateChange(state: state)
    }
    
    
    func updataActionInfoModelView(model:TSActionInfoModel){
        
        if model.modelType == .example {
            model.actionStatus = .success
        }
        
        switch model.actionStatus {
        case .pending,.running:
            generateView.isHidden = false
            generateView.setProgress(progress: model.percent)
        case .success:
            generateView.isHidden = true
            
            ringView.timeLab.text = Float(model.request.duration).floatToMinuteSecond()
            ringView.nameLab.text = model.response.title
            ringView.setCoverImageView(urlString: model.response.coverUrl)
            exampleView.isHidden = model.modelType != .example
            vipView.isHidden = true
            
            self.changePlayerState(state: TSBusinessAudioPlayer.shared.currentPlayerState)
            
        case .failed:
            generateView.isHidden = false
            generateView.setFail()
        }
    }

    var playSelf:Bool{
        return TSBusinessAudioPlayer.shared.isPlayURLString(string: modelUrlString,localURL: modelLocalURL,indexPath: indexPath)
    }

    var modelUrlString:String{
        var urlString:String = ""
        if let model = model{
            urlString = model.response.musicUrl
        }else if let ringModel = ringModel{
            urlString = ringModel.audioUrl
        }
        return urlString
    }

    var modelLocalURL:URL?{
        if let model = model{
            return nil
        }else if let ringModel = ringModel{
            return TSDownloadManager.getRingLocalURL(ringModel: ringModel)
        }
        return nil
    }
    
}
//点击编辑和设置按钮
extension TSAIRintoneHistoryCell {
    
    @objc func clickSetRingBtn(){
        if let model = model {
            setShareBand(infoModel: model)
        }else if let ringModel = ringModel  {
            setShareBand(ringModel: ringModel)
        }
    
        TSBusinessAudioPlayer.shared.stop()
    }
    
    func setShareBand(infoModel:TSActionInfoModel){
        guard let targetVC = targetVC else { return }
        let needVip = infoModel.response.vip
        let urlString = infoModel.response.musicUrl
        let fileName = infoModel.response.title

        TSDownloadManager.getDownLoadRing(urlString: urlString){ url,downLoad in
            if let path = url {
                _ = kPurchaseToolShared.kshareBand(needVip: needVip, vc: targetVC, fileURL: path, fileName: fileName)
            }
        }
    }
    
    func setShareBand(ringModel:TSRingModel){
        guard let targetVC = targetVC else { return }
        let needVip = ringModel.vip
        let fileName = ringModel.title
        
        let kshareBand:(URL)->Void = { path in
            _ = kPurchaseToolShared.kshareBand(needVip: needVip, vc: targetVC, fileURL: path, fileName: fileName){  success in
                if success {
                    var documentPath = path.path
                    if path.path.isDocumentPath == false{
                        let savePath = TSBusinessFileManager.saveRingPathURL.appendingPathComponent(path.lastPathComponent)
                        TSFileManagerTool.copyFileWithOverwrite(from: path, to: savePath)
                        documentPath = savePath.path
                    }
                    ringModel.documentPath = documentPath.documentLastURLString
                    TSMineRintoneHistory.shared.replaceAndSaveModel(saveModel: ringModel) { ringModel, model in
                        if model.title == ringModel.title,
                           model.audioUrl == ringModel.audioUrl,
                           model.categoryId == ringModel.categoryId,
                           model.documentPath == ringModel.documentPath{
                            return true
                        }
                        return false
                    }
                }
            }
        }
    
        TSDownloadManager.getDownLoadRing(ringModel: ringModel) { url, downLoad in
            if let path = url {
                kshareBand(path)
            }
        }
    }
    
    @objc func clickEidtBtn(){

        guard let targetVC = targetVC else { return }
        var editRingModel:TSRingModel? = nil
        if let model = self.ringModel {
            editRingModel = model
        }else if let model = self.model {
            editRingModel = TSRingModel.getTSRingModel(TSActionInfoModel: model)
        }

        if let editRingModel = editRingModel,let copyRingModel = editRingModel.copy() as? TSRingModel{
            let urlString = copyRingModel.audioUrl
            TSDownloadManager.getDownLoadRing(ringModel: copyRingModel)
            { [weak self] url, downLoad in
                guard let self = self else { return }
                if let path = url {
                    pushVCTSEditAudioVideo(targetVC: targetVC, modelVC: TSEditAudioVC(ringModel: copyRingModel, editOriginalURL: path))
                }else{
                    kShowToastDataMissing()
                }
            }
        }
    }
    
    func pushVCTSEditAudioVideo(targetVC:UIViewController,modelVC:TSEditAudioVC){
    
        if let fileInfo = TSBusinessAudioPlayer.getAudioFileInfo(path: modelVC.editOriginalURL.path) {
            if let size = fileInfo.sizeInBytes {
                modelVC.ringModel.size = Int(size)
            }
            
            if let duration = fileInfo.durationInSeconds {
                modelVC.ringModel.duration = Int(duration)
            }
        }
        
        TSBusinessAudioPlayer.shared.stop()
        kPushVC(target: targetVC, modelVC: modelVC)
    }
}
