//
//  TSEditAudioVideoBaseVC.swift
//  AIRingtone
//
//  Created by 100Years on 2025/3/26.
//
import AVFoundation
class TSEditAudioVideoBaseVC: TSBaseVC {
    //#################################### 数据区域 ####################################//
    var titleName:String{
        "Edit".localized
    }
    var editOriginalURL:URL
    
    init(editOriginalURL:URL) {
        self.editOriginalURL = editOriginalURL
        super.init()
    }
    
    @MainActor required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    //#################################### TSCustomStackView ####################################//
    lazy var cusStackView: TSCustomStackView = {
        let cusStackView = TSCustomStackView(axis: .vertical,spacing: 0)
        return cusStackView
    }()
    

    //#################################### play按钮 ####################################//
    
    lazy var playButtonView: UIView = {
        let playButtonView = UIView(frame: CGRectMake(0, 0, k_ScreenWidth, 18+80+18))
        
        let playButtonBg = UIView()
        playButtonBg.backgroundColor = .white.withAlphaComponent(0.1)
        playButtonBg.cornerRadius = 40
        playButtonView.addSubview(playButtonBg)
        playButtonBg.snp.makeConstraints { make in
            make.width.height.equalTo(80)
            make.center.equalToSuperview()
        }
        
        playButtonBg.addSubview(playButton)
        playButton.snp.makeConstraints { make in
            make.width.height.equalTo(60)
            make.center.equalToSuperview()
        }
        
        return playButtonView
    }()
    
    lazy var playButton: UIButton = {//editaudio_play //editaudio_pause
        let playButton = UIButton.createButton(image: UIImage(named: "editaudio_play")){ [weak self]  in
            guard let self = self else { return }
            startPlay()
        }
        playButton.setImage(UIImage(named: "editaudio_pause"), for: .selected)
        return playButton
    }()
    
    //#################################### 底部保存按钮等 ####################################//
    lazy var cutButton: TSAppBtnView = { //保存
        let cutButton = TSAppBtnView()
        cutButton.setUpButton(style: .normalBorder, btnFrame: CGRectMake(0, 0, 98*kDesignScale, 48)) { [weak self]  in
            guard let self = self else { return }
            saveButtonClick()
        }
        cutButton.button.setTitle("Save".localized, for: .normal)
        return cutButton
    }()
    
    lazy var doneButton: TSAppBtnView = { // set
        let doneButton = TSAppBtnView()
        doneButton.setUpButton(style: .normalSet, btnFrame: CGRectMake(0, 0, 233*kDesignScale, 48)) { [weak self]  in
            guard let self = self else { return }
            setButtonClick()
        }
        doneButton.button.setTitle("Set Now".localized, for: .normal)
        return doneButton
    }()
    
    lazy var bottomBtnView: UIView = {
        let bottomBtnView = UIView()
        
        bottomBtnView.addSubview(cutButton)
        bottomBtnView.addSubview(doneButton)
        
        cutButton.snp.makeConstraints { make in
            make.top.equalTo(16)
            make.bottom.equalTo(-16)
            make.leading.equalTo(16)
            make.size.equalTo(cutButton.button.size)
        }
        
        doneButton.snp.makeConstraints { make in
            make.top.equalTo(16)
            make.bottom.equalTo(-16)
            make.trailing.equalTo(-16)
            make.size.equalTo(doneButton.button.size)
        }
        return bottomBtnView
    }()
    
    //#################################### viewDidLoad####################################//
    override func createData() {

    }
    
    override func createView() {
        addNormalNavBarView()
        setPageTitle(titleName)
        
        contentView.addSubview(bottomBtnView)
        bottomBtnView.snp.makeConstraints { make in
            make.leading.trailing.equalTo(0)
            make.bottom.equalTo(-k_Height_safeAreaInsetsBottom())
        }
        
        contentView.addSubview(cusStackView)
        cusStackView.snp.makeConstraints { make in
            make.top.leading.bottom.trailing.equalTo(0)
            make.bottom.equalTo(bottomBtnView.snp.top)
        }
    }

    lazy var operationCache: [TSRingModel] = []
    var tempMp3Path: String?
    var needClearTemp : Bool = false

    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)

        if needClearTemp {
            deleteTempFiles()
        }
    }
    
    func deleteTempFiles() {
        if let tempMp3Path = tempMp3Path {
            let mp3Dir = NSString(string: tempMp3Path).deletingLastPathComponent
            if FileManager.default.isDeletableFile(atPath: mp3Dir) {
                let fileUrl = URL(fileURLWithPath: mp3Dir)
                do {
                    try FileManager.default.removeItem(at: fileUrl)
                } catch {
                    dePrint("TSEditAudioVideoBaseVC 删除文件失败")
                }
            }
        }
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        // 恢复右滑返回手势
        navigationController?.interactivePopGestureRecognizer?.isEnabled = true
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        // 禁用右滑返回手势
        navigationController?.interactivePopGestureRecognizer?.isEnabled = false

    }
    
    deinit {
        dePrint("RingEditViewController 销毁了")
    }
    
    override func navBarClickLeftAction() {
        TSCustomAlertController.show(in: self, config: TSCustomAlertController.AlertConfig(
            message: "As you leave, the changes you have made will be lost.".localized,
            messageColor: .white,
            messageFont: .systemFont(ofSize: 16),
            
            cancelTitle: "Leave".localized,
            cancelColor: .white,
            
            confirmTitle: "Stay".localized,
            confirmColor: .themeColor,
            
            cancelAction: {
                print("用户点击了Leave")
                super.navBarClickLeftAction()
            },
            confirmAction: {
                print("用户点击了Stay")
            }
        ))
    }
    
    
    
    func startPlay() {
        
    }
    
    func saveButtonClick() {
        
    }
    
    func setButtonClick() {
        
    }
}
